//--------------------------------------------------------------
// Parallel World Library
//--------------------------------------------------------------
#ifndef _PWLIB_H_
#define _PWLIB_H_

#ifdef _WIN64
	#pragma comment( lib, "pwlib_x64.lib" )
#else
	#pragma comment( lib, "pwlib.lib" )
#endif

#ifdef __cplusplus
#define PWLIB_IMPORT extern "C" __declspec(dllimport)
#else
#define PWLIB_IMPORT __declspec(dllimport)
#endif

//--------------------------------------------------------------
// constant
//--------------------------------------------------------------
#define PWLIB_UNIT_SAMPLE			(0)
#define PWLIB_UNIT_MS				(1)

#define PWLIB_CNVOPTION_PIT_MODE1	(0x00)
#define PWLIB_CNVOPTION_PIT_MODE2	(0x01)

#define PWLIB_PRM_VOLUME			(0)
#define PWLIB_PRM_PAN				(1)
#define PWLIB_PRM_MUTE_FLAG			(2)
#define PWLIB_PRM_SOLO_FLAG			(3)
#define PWLIB_PRM_OFFSET			(4)

#define PWLIB_MAIN_PREFIX			(0)
#define PWLIB_MAIN_SUFFIX			(1)
#define PWLIB_MORPHING_PREFIX		(2)
#define PWLIB_MORPHING_SUFFIX		(3)

//--------------------------------------------------------------
// Error code
//--------------------------------------------------------------
#define PWLIB_ERROR_NOERROR			(0x00)
#define PWLIB_ERROR_UNSUPPORTED		(0x01)

#define PWLIB_ERROR_ARGUMENT		(0x10)
#define PWLIB_ERROR_NO_MEMORY		(0x11)
#define PWLIB_ERROR_ALREADY_OPENED	(0x12)
#define PWLIB_ERROR_ALREADY_CLOSED	(0x13)

#define PWLIB_ERROR_OPEN_VOICE_DIR	(0x20)
#define PWLIB_ERROR_OPEN_OTO_FILE	(0x21)
#define PWLIB_ERROR_OPEN_MAP_FILE	(0x22)
#define PWLIB_ERROR_OPEN_WAV_FILE_W	(0x23)
#define PWLIB_ERROR_OPEN_WAV_FILE_R	(0x24)
#define PWLIB_ERROR_OPEN_UST_FILE	(0x25)
#define PWLIB_ERROR_OPEN_FRQ_FILE_W	(0x26)
#define PWLIB_ERROR_OPEN_FRQ_FILE_R	(0x27)
#define PWLIB_ERROR_PROGRAM_FILES	(0x28)

#define PWLIB_ERROR_WRITE_WAV_FILE	(0x30)
#define PWLIB_ERROR_WRITE_FRQ_FILE	(0x31)

#define PWLIB_ERROR_WAV_FORMAT		(0x40)
#define PWLIB_ERROR_FRQ_FORMAT		(0x41)
#define PWLIB_ERROR_UST_FORMAT		(0x42)
#define PWLIB_ERROR_OTO_UTF8		(0x43)
#define PWLIB_ERROR_UST_UTF8		(0x44)

#define PWLIB_ERROR_APPEND_NUM_OVER	(0x50)

#define PWLIB_ERROR_WAVE_OUT		(0x60)

//--------------------------------------------------------------
// Structure
//--------------------------------------------------------------

//--------------------------------------------------------------
// Handle
//--------------------------------------------------------------
typedef struct PWLIB_HPROJECT__ { int unused; } *PWLIB_HPROJECT;
typedef struct PWLIB_HVOICETRACK__ { int unused; } *PWLIB_HVOICETRACK;
typedef struct PWLIB_HVOICE__ { int unused; } *PWLIB_HVOICE;

//--------------------------------------------------------------
// Function
//--------------------------------------------------------------
#ifdef _UNICODE
#define	PWLibLoadWaveFile	PWLibLoadWaveFileW
#define	PWLibExportWaveFile	PWLibExportWaveFileW
#define	PWLibLoadUstFile	PWLibLoadUstFileW
#else
#define	PWLibLoadWaveFile	PWLibLoadWaveFileA
#define	PWLibExportWaveFile	PWLibExportWaveFileA
#define	PWLibLoadUstFile	PWLibLoadUstFileA
#endif

// Library
PWLIB_IMPORT int __stdcall PWLibGetVersion( void );

// Project
PWLIB_IMPORT int __stdcall PWLibCreateProject( PWLIB_HPROJECT *hProject );
PWLIB_IMPORT int __stdcall PWLibDeleteProject( PWLIB_HPROJECT hProject );

PWLIB_IMPORT int __stdcall PWLibLoadWaveFileA( PWLIB_HPROJECT hProject, int trackNum, const char *fileName );
PWLIB_IMPORT int __stdcall PWLibLoadWaveFileW( PWLIB_HPROJECT hProject, int trackNum, const wchar_t *fileName );
PWLIB_IMPORT int __stdcall PWLibUnloadWaveFile( PWLIB_HPROJECT hProject, int trackNum );

PWLIB_IMPORT int __stdcall PWLibLoadUstFileA( PWLIB_HPROJECT hProject, int trackNum, const char *fileName, const char *voiceRootPath );
PWLIB_IMPORT int __stdcall PWLibLoadUstFileW( PWLIB_HPROJECT hProject, int trackNum, const wchar_t *fileName, const wchar_t *voiceRootPath );
PWLIB_IMPORT int __stdcall PWLibUnloadUstFile( PWLIB_HPROJECT hProject, int trackNum );

PWLIB_IMPORT int __stdcall PWLibSetProjectPrm( PWLIB_HPROJECT hProject, int prmNum, double prm );
PWLIB_IMPORT int __stdcall PWLibGetProjectPrm( PWLIB_HPROJECT hProject, int prmNum, double *prm );
PWLIB_IMPORT int __stdcall PWLibSetVoiceTrackPrm( PWLIB_HPROJECT hProject, int trackNum, int prmNum, double prm );
PWLIB_IMPORT int __stdcall PWLibGetVoiceTrackPrm( PWLIB_HPROJECT hProject, int trackNum, int prmNum, double *prm );
PWLIB_IMPORT int __stdcall PWLibSetAudioTrackPrm( PWLIB_HPROJECT hProject, int trackNum, int prmNum, double prm );
PWLIB_IMPORT int __stdcall PWLibGetAudioTrackPrm( PWLIB_HPROJECT hProject, int trackNum, int prmNum, double *prm );

// Track
PWLIB_IMPORT int __stdcall PWLibGetVoiceTrackHandle( PWLIB_HPROJECT hProject, int trackNum, PWLIB_HVOICETRACK *hTrack );
PWLIB_IMPORT int __stdcall PWLibSetMorphingPrefixA( PWLIB_HVOICETRACK hTrack, int type, const char *prefix );
PWLIB_IMPORT int __stdcall PWLibSetMorphingPrefixW( PWLIB_HVOICETRACK hTrack, int type, const wchar_t *prefix );
PWLIB_IMPORT int __stdcall PWLibConvertVoiceTrack( PWLIB_HVOICETRACK hTrack, unsigned int option );

// Frq
PWLIB_IMPORT int __stdcall PWLibStartCreatingFrqFile( PWLIB_HPROJECT hProject );
PWLIB_IMPORT int __stdcall PWLibWaitCreatingFrqFile( PWLIB_HPROJECT hProject );
PWLIB_IMPORT int __stdcall PWLibStopCreatingFrqFile( PWLIB_HPROJECT hProject );

// Synth
PWLIB_IMPORT int __stdcall PWLibOpenWaveOut( PWLIB_HPROJECT hProject, int deviceId );
PWLIB_IMPORT int __stdcall PWLibCloseWaveOut( PWLIB_HPROJECT hProject );
PWLIB_IMPORT int __stdcall PWLibWoPlay( PWLIB_HPROJECT hProject );
PWLIB_IMPORT int __stdcall PWLibWoStop( PWLIB_HPROJECT hProject );
PWLIB_IMPORT int __stdcall PWLibWoSeek( PWLIB_HPROJECT hProject, int unit, int offset );
PWLIB_IMPORT int __stdcall PWLibWoGetStatus( PWLIB_HPROJECT hProject, int *status );
PWLIB_IMPORT int __stdcall PWLibWoGetPosition( PWLIB_HPROJECT hProject, int unit, int *position );
PWLIB_IMPORT int __stdcall PWLibGetProjectLength( PWLIB_HPROJECT hProject, int unit, int *length );
PWLIB_IMPORT int __stdcall PWLibExportWaveFileA( PWLIB_HPROJECT hProject, const char *fileName, int bitDeapth, int nChannel );
PWLIB_IMPORT int __stdcall PWLibExportWaveFileW( PWLIB_HPROJECT hProject, const wchar_t *fileName, int bitDeapth, int nChannel );
PWLIB_IMPORT int __stdcall PWLibSynthesize( PWLIB_HPROJECT hProject, int index, int size, double *waveData );

#endif /* _PWLIB_H_ */
